; ******************************
; ZX80 Breakout - Loader Program
; ******************************
; (c)1980 Macronics
;
; RAM required: 1K
;
; Disassembled by Paul Farrow, www.fruitcake.plus.com
; 11th March 2011 (revised 16th April 2011)


; =====
; Notes
; =====

; This disassembly listing uses tab characters with tab stops at every 8th column.

; This program is loaded into RAM at 17322 ($43AA) and is then run. It scans through the
; BASIC program finding all PRINT strings, decoding the hex bytes and storing them in RAM
; beginning at $4000. The mechanism stops when a line number with a low byte of $00 is found.
; This will be line 300 of the program.


; ====================
; Assembler Directives
; ====================

#define DEFB .BYTE
#define DEFW .WORD
#define DEFM .TEXT
#define DEFS .FILL
#define EQU  .EQU
#define ORG  .ORG
#define END  .END

; ======================================================================================================================================================

        ORG  $43AA

L43AA:  LD   DE,$4000           ; The destination address for the main program code.
        LD   HL,$4028           ; The start of the BASIC program area.

; Enter a loop to process each line.

L43B0:  LD   A,(HL)             ; Has a line number with high byte of $00 been found, i.e. reached line 300?
        AND  A
        JP   NZ,$400E           ; Jump if so to run the routine now in RAM at $400E.

; The low byte of the line number is not $00 and so there are still PRINT lines to process.

L43B5:  INC  HL                 ; Fetch the next character.
        LD   A,(HL)
        CP   $01                ; Has the opening quote been found?
        JR   NZ,L43B5           ; Jump back until

; The start of the PRINT string has been found so fetch the first character.

        INC  HL
        LD   A,(HL)             ; Fetch the high nibble.

; Enter a loop to process the hex characters from a single line.

L43BD:  SUB  $1C                ; Convert to a numerical value, i.e. sub '0'.
        RLCA
        RLCA
        RLCA
        RLCA                    ; High nibble*16.
        LD   B,A

        INC  HL
        LD   A,(HL)             ; Fetch the low nibble.
        SUB  $1C                ; Convert to a numerical value, i.e. sub '0'.
        ADD  A,B                ; Add the high nibble to form a byte value.

        LD   (DE),A             ; Store the byte value.

        INC  DE                 ; Next destination address.
        INC  HL                 ; Next source address.

        LD   A,(HL)             ; Fetch the next character.
        CP   $01                ; Is it the terminating quote?
        JR   NZ,L43BD           ; Jump back if not to process the next character.

; A line has been fully processed to check whether there is another line to be handled.

L43D1:  INC  HL                 ; Next source address.
        LD   A,(HL)
        CP   $76                ; Has the end of the line been reached?
        JR   NZ,L43D1           ; Loop back if not until it is found.

        INC  HL                 ; Advance to the start of the next line.
        JR   L43B0              ; Process the new line.

; ======================================================================================================================================================

        END

